// Copyright 2000-2023 JetBrains s.r.o. and contributors. Use of this source code is governed by the Apache 2.0 license.
package com.intellij.collaboration.ui.codereview.details.model

import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.MutableStateFlow
import kotlinx.coroutines.flow.StateFlow
import kotlinx.coroutines.flow.asSharedFlow

interface CodeReviewChangesViewModel<T> {
  val reviewCommits: Flow<List<T>>
  val selectedCommit: Flow<T?>

  /** `-1` for "all commits" mode */
  val selectedCommitIndex: Flow<Int>

  fun selectCommit(index: Int)

  fun selectNextCommit()

  fun selectPreviousCommit()

  fun commitHash(commit: T): String
}

abstract class CodeReviewChangesViewModelBase<T> : CodeReviewChangesViewModel<T> {
  abstract override val reviewCommits: StateFlow<List<T>>

  private val _selectedCommitState: MutableStateFlow<T?> = MutableStateFlow(null)
  override val selectedCommit: Flow<T?> = _selectedCommitState.asSharedFlow()

  private val _selectedCommitIndexState: MutableStateFlow<Int> = MutableStateFlow(-1)
  override val selectedCommitIndex: Flow<Int> = _selectedCommitIndexState.asSharedFlow()

  override fun selectCommit(index: Int) {
    _selectedCommitState.value = reviewCommits.value.getOrNull(index)
    _selectedCommitIndexState.value = index
  }

  override fun selectNextCommit() {
    _selectedCommitIndexState.value++
    _selectedCommitState.value = reviewCommits.value[_selectedCommitIndexState.value]
  }

  override fun selectPreviousCommit() {
    _selectedCommitIndexState.value--
    _selectedCommitState.value = reviewCommits.value[_selectedCommitIndexState.value]
  }
}